# Copyright (c) 2014  Dave Love, University of Liverpool
# Licence as for the package itself

# The packaging mostly follows Debian's.  rpmlint complains about the
# library sonames although they fit what "rpmlint -Iinvalid-soname"
# says.
# NB.  This is not the latest version, but it's not yet clear if that's
# compatible with PETSc et al.

%if 0%{?el6}
%ifarch ppc64
%bcond_with mpich
%else
%bcond_without mpich
%endif
%else
%bcond_without mpich
%endif

Name:           hypre
Version:        2.8.0b
Release:        16%{?dist}
Summary:        High performance matrix preconditioners
License:        LGPLv2
URL:            https://computation-rnd.llnl.gov/linear_solvers/
# (Not directly accessible) from
# https://computation-rnd.llnl.gov/linear_solvers/software.php
Source:         %{name}-%{version}.tar.xz
# These patches are from Debian
# Resolve circular dependency
Patch1:         hypre-flexgmres-to-parcsr_ls.patch
# Get all libraries installed
Patch2:         hypre-install-libs.patch
# Resolve circular dependency
Patch3:         hypre-fgmres-lib.patch
# Modify linking (see file header)
Patch4:         hypre-shlibs-interlink.patch

BuildRequires:  gcc-c++ gcc-gfortran automake libtool libtool-ltdl-devel
BuildRequires:  openmpi-devel SuperLU-devel
%if %{with mpich}
BuildRequires:  mpich-devel
%endif
%ifarch %{ix86} x86_64
BuildRequires:  openblas-devel
%else
BuildRequires:  atlas-devel
%endif
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%global __requires_exclude libHYPRE.*
%{?filter_setup:
%filter_from_requires /libHYPRE/d
%filter_setup
}

%global desc \
Hypre is a set of matrix preconditioning libraries to aid in the\
solution of large systems of linear equations.

%description
%desc

%package devel
Summary:        Development files for %name
Requires:       hypre%{?_isa} = %{version}
Requires:       SuperLU-devel%{?_isa} openblas-devel%{?_isa}

%description devel
Development files for %name

%package openmpi
Summary:        High performance matrix preconditioners - openmpi
Requires:       openmpi%{?_isa}

%description openmpi
%desc

This is the openmpi version.

%package openmpi-devel
Summary:        Development files for %name-openmpi
Requires:       hypre-openmpi%{?_isa} = %{version}-%{release}
Requires:       openmpi-devel%{?_isa} SuperLU-devel%{?_isa}
Requires:       openblas-devel%{?_isa}

%description openmpi-devel
Development files for %name-openmpi

%if %{with mpich}
%package mpich
Summary:        High performance matrix preconditioners - mpich
Requires:       mpich%{?_isa}

%description mpich
%desc

This is the mpich version.

%package mpich-devel
Summary:        Development files for %name-mpich
Requires:       hypre-mpich%{?_isa} = %{version}-%{release}
Requires:       mpich-devel%{?_isa} SuperLU-devel%{?_isa}
Requires:       openblas-devel%{?_isa}

%description mpich-devel
Development files for %name-mpich
%endif

%package doc
Summary:        Documentation for hypre
BuildArch:      noarch

%description doc
%desc

This is the documentation package.

%prep
%setup -q

%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1

rm -r src/{blas,lapack}

find \( -name \*.[ch] -o -name \*.cxx \) -perm /=x -exec chmod 0644 {} \;

(cd src; config/bootstrap)
ln -s ../krylov/HYPRE_flexgmres.c ../krylov/flexgmres.c src/parcsr_ls/
(cd src/babel-runtime && libtoolize -f)
cp -f src/babel-runtime/libltdl/ltdl.* src/babel-runtime/sidl/
rm -f src/babel-runtime/config/config.*
(cd src/babel-runtime && aclocal -I m4 && automake -a)
# configure doesn't work out-of-tree 
cp -a src openmpi
%if %{with mpich}
cp -a src mpich
%endif

%build

%ifarch %{ix86} x86_64
%global lalibs --with-blas-libs=openblas --with-lapack-libs=openblas
%else
%global lalibs --with-blas="-L%{_libdir}/atlas -lf77blas -latlas" --with-lapack="-L%{_libdir}/atlas -llapack -latlas"
%endif

export LDFLAGS='-Wl,--as-needed'
pushd src
%configure --without-examples --without-MPI --with-timing --without-openmp \
           %lalibs \
           --with-lapack --enable-shared --without-superlu
make %{?_smp_mflags}
popd

%global do_mpi_build \
%configure --prefix=$MPI_HOME --without-examples \\\
           --with-MPI --with-MPI-include=$MPI_INCLUDE \\\
           --with-MPI-lib-dirs=$MPI_lib --with-timing --without-openmp \\\
           %lalibs \\\
           --with-lapack --enable-shared --without-superlu \\\
           CC=mpicc CXX=mpicxx F77=mpif77 \
make %{?_smp_mflags}

pushd openmpi
%_openmpi_load
%do_mpi_build
%_openmpi_unload
popd

%if %{with mpich}
pushd mpich
%_mpich_load
%do_mpi_build
%_mpich_unload
popd
%endif


%install

make -C src install HYPRE_INSTALL_DIR=%{buildroot}%{_prefix} \
     HYPRE_LIB_INSTALL=%{buildroot}%{_libdir} \
     HYPRE_INC_INSTALL=%{buildroot}%{_includedir}/%{name}

%_openmpi_load
make -C openmpi install HYPRE_INSTALL_DIR=%{buildroot}$MPI_HOME \
     HYPRE_LIB_INSTALL=%{buildroot}$MPI_LIB \
     HYPRE_INC_INSTALL=%{buildroot}$MPI_INCLUDE/%{name}
%_openmpi_unload

%if %{with mpich}
%_mpich_load
make -C mpich install HYPRE_INSTALL_DIR=%{buildroot}$MPI_HOME \
     HYPRE_LIB_INSTALL=%{buildroot}$MPI_LIB \
     HYPRE_INC_INSTALL=%{buildroot}$MPI_INCLUDE/%{name}
%_mpich_unload
%endif

%post -n hypre -p /sbin/ldconfig
%postun -n hypre -p /sbin/ldconfig


%check
%_openmpi_load
make -C openmpi check


%files -n hypre
%doc CHANGELOG COPYING.LESSER COPYRIGHT src/docs/copyright.txt
%{_libdir}/*-%{version}.so

%files devel
%exclude %{_libdir}/*-%{version}.so
%{_libdir}/*.so
%{_includedir}/%{name}

%files openmpi
%doc CHANGELOG COPYING.LESSER COPYRIGHT src/docs/copyright.txt
%{_libdir}/openmpi/lib/*-%{version}.so

%files openmpi-devel
%exclude %{_libdir}/openmpi/lib/*-%{version}.so
%{_libdir}/openmpi/lib/*.so
%{_includedir}/openmpi-%_arch/%{name}

%if %{with mpich}
%files mpich
%doc CHANGELOG COPYING.LESSER COPYRIGHT src/docs/copyright.txt
%{_libdir}/mpich/lib/*-%{version}.so

%files mpich-devel
%exclude %{_libdir}/mpich/lib/*-%{version}.so
%{_libdir}/mpich/lib/*.so
%{_includedir}/mpich-%_arch/%{name}
%endif

%files doc
%doc CHANGELOG COPYING.LESSER COPYRIGHT src/docs/copyright.txt
%doc src/docs/*.pdf
%doc src/docs/*.html
%doc src/docs/*.gif

%changelog
* Fri Feb 20 2015 Dave Love <d.love@liverpool.ac.uk> - 2.8.0b-16
- Conditionalize out mpich on ppc64 el6
- Use -f with libtoolize (for f22)
- Fix some missing _isa in requires
- Drop unnecessary configure patch
- Run tests
- Small simplifications

* Wed Feb  4 2015 Dave Love <d.love@liverpool.ac.uk> - 2.8.0b-15
- Fix requires

* Mon Dec 29 2014 Dave Love <d.love@liverpool.ac.uk> - 2.8.0b-15
- Build without internal superlu

* Thu Dec 11 2014 Dave Love <d.love@liverpool.ac.uk> - 2.8.0b-14
- mpich version

* Fri Nov 28 2014 Dave Love <d.love@liverpool.ac.uk> - 2.8.0b-13
- Initial packaging, following Debian, including patches
  High release number to avoid clash with old SuSE-based version
